# -*- coding: utf-8 -*-
"""PasantiaUIS.ipynb

Automatically generated by Colaboratory.

Original file is located at
    https://colab.research.google.com/drive/1FNpizbc0lU29KOe8DCKE4V7kO2Fi6VKp

Para correr en colab el siguiente Notebook se debe copiar la siguiente carperta a su drive https://drive.google.com/drive/folders/1WIAhHnISIC73b1Uu81Hp0ryvB5FYM6rB?usp=sharing

O descargar la data contenida en el mismo y modificar las rutas de acceso
"""

#@title **MONTAR EL DRIVE** (Necesario en Colab, de lo contrario comentar){ display-mode: "form" }
import os
#from google.colab import drive

#path_ai = "Pasantias UIS" #@param {type:"string"}
#path = "/content/drive/My Drive/" + path_ai
#drive.mount('/content/drive')
#os.chdir(path)
print('Usted esta trabajando en el siguiente directorio:', os.getcwd())

#@title **O Subir Archivos**  (Descomentar de ser Necesario) { display-mode: "form" }
#from google.colab import files 
#files.upload()

"""# Torres de Centelleo (UIS)

El siguiente Notebook contiene recursos desarrollados para trabajar con los páneles centelladores del Grupo Halley de la Universidad Industrial de Santander.

El equipo de medición esta conformado por los siguientes componentes

- Centelladores EJ-200,  Eljen Technology.
- PM de Silicio: MicroFC-60035-SMT, SensL Technologies, Ltd.
- QuarkNet 6000 Data Acquisition (DAQ) board.
- Raspberry Pi 2 Modelo B.

## Modulos y funciones
"""

#@title **Load libs** { display-mode: "form" }
import csv, operator
import numpy as np
import pandas as pd
import scipy.stats as stats
import inspect
import matplotlib.pyplot as plt
import seaborn as sns
from scipy import stats
from datetime import datetime
from pandas import DataFrame as df
from matplotlib.axes._axes import _log as matplotlib_axes_logger
from scipy.optimize import curve_fit
from scipy.special import factorial
from scipy.stats import poisson
matplotlib_axes_logger.setLevel('ERROR')
#sns.set_theme()
pd.__version__

#@title **Función Lectura** (Comando CE) { display-mode: "form" }
#El comando CE dispone en pantalla cada uno de los eventos registrados por lo que hace falta buscar las coincidencias
#en los centelladores agrupandolas por conteos del reloj
# Las salidas son: El promedio de conteos por segundo, la desviacion Estandar y el arreglo de conteos 
def data_analysis(ruta):
    data = pd.read_csv(ruta, delimiter=' ', comment='S',
                   names = ['T1', 'CH1a', 'CH1b', 'CH2a', 'CH2b', 'CH3a', 'CH3b', 'CH4a', 'CH4b', 'T2', 'hora', 'fecha', 'rec1', 'rec2', 'rec3', 'rec4'], 
                   parse_dates={'tiempo sin convertir': [10]})

    data['HORA']= pd.to_datetime(data['tiempo sin convertir'],format='%H%M%S.%f') 
    fn = np.unique(data['HORA']) 
    frec = []

    for i in fn:
        frec.append(np.sum(data['HORA']==i))
    frec = np.array(frec)
    data2 = pd.DataFrame({'fecha': fn, 'frec': frec})
    data2.describe().transpose()

    mean = np.mean(frec)
    std = np.std(frec)

    return(mean,std,frec)

    #print("Mean rate : %.2f +/- %.2f" %(mean, std))

#@title **Función de Lectura** Comando (ST 3 1) { display-mode: "form" }
# El comando ST imprime el estado actual de los 3 canales individualmente y el contador de coincidencias 
# registradas por DAQ, ademas cada 5 min ejecuta el comando DS que imprime los estados actuales del sistema de detección. 
# por lo que se busca separar las salidas de ambos comandos (ST y DS)
# La función de lectura tiene por argumento la ruta y la fecha de inicio de las mediciones.(Mejorar posteriormente)
# El valor de Skiprows depende del head del formato de salida de los datos. 

def lectura(ruta,fecha):
    df=pd.read_table(ruta,delim_whitespace=True, skiprows=53,names=["Command","mBar", "GpsDegC", "1ppsDly", "CPU_Vcc", "GpsUTC","GpsDate", "GpsVld", "GpsSat#","1ppsTime", "CodeVer", "SerNum","TMCregs", "Cntlregs"],dtype=object)
    df=df[:-46] ## Por el tail del formato de salida
    df_mask = df['Command']=='DS'
    filtered_df = df[df_mask]
    filtered_df = filtered_df.drop(['GpsDate','GpsVld', 'GpsSat#', '1ppsTime', 'CodeVer', 'SerNum', 'TMCregs','Cntlregs'], axis=1)
    filtered_df.set_index('Command',inplace = True)

    a,b = filtered_df.shape
    matrix = np.zeros((a,b))
    for i in range(a):
        for j in range(5):
            matrix[i][j] = int(filtered_df.iloc[i,j],base = 16)

    df = pd.DataFrame(matrix, index = filtered_df.index, columns=filtered_df.columns)
    df.rename(columns = {'mBar':'Ch0Cnts','GpsDegC':'Ch1Cnts','1ppsDly':'Ch2Cnts', 'CPU_Vcc':'Ch3Cnts', 'GpsUTC':'TrigCnts'}, inplace = True) #Nombre de los comando DS (Renombrar)
    idx = pd.date_range(fecha, periods = len(df), freq ='min') 
    df.index=idx
    df['time']=idx
    
    return df

#@title **Función de Lectura** (Datos Metereológicos Davis) { display-mode: "form" }
# El comando ST imprime el estado actual de los 3 canales individualmente y el contador de coincidencias 
def lecturaMet(ruta):
  datap=pd.read_table(ruta,skiprows=2,names=['Date',	'Time',	'Out',	'HiTemp',	' Low Temp',	'Hum',	'Pt.',	'Speed',	'Dir',	'Run',	'Speed2',	'Dir2',	'Chill',	'Index',	'Index2',	'Bar',  	'Rain',	'Rate',	'D-D', 	'D-D2', 	'Temp4',	'Hum2',	'Dew',	'Heat',	'EMC',	'Density',	'Samp',	'Tx', 	'Recept',	'Int.'])
  b = datap['Date']+'_'+datap['Time']
  datap['Hora'] = pd.to_datetime(b,format='%d/%m/%y_%H:%M')
  datap.index=datap['Hora']
  return datap

#@title Funcion $\chi^2$

def chi_cuadrado(observed, expected):
    return sum([((o - e)**2)/e for o, e in zip(observed, expected)])

"""## Calibración del treshold"""

#@title **Lectura de los Datos** (Calibracion) { display-mode: "form" }
a=os.listdir('Datos/Calibracion') #lista de archivos de calibracion, modificar a la ruta de los datos
a.sort() #Ordena segun nombre del archivo
lista=[]
for i in a:
  lista.append(data_analysis('Datos/Calibracion/'+i)[0:2]) #Lectura

#@title **Calibración Graficación** { display-mode: "form" }
sns.set(rc = {'figure.figsize':(6,4)})
df=pd.DataFrame(lista)
df['tr']=[5,10,15,20,25,30,35,40,45,50] #Voltajes de Treshold Usados
df.columns=['Conteos','std','tresh']
df.plot(x='tresh', y='Conteos', yerr='std', kind='scatter',fontsize=16,linewidth=4)
plt.ylabel('Flujo en el Detector, $[Conteos/s]$',size=16)
plt.xlabel('Threshold, $[mV]$',size=16)
plt.title('Calibración Threshold', size=20)
plt.show()

"""## Modulaciones del Flujo"""

#@title **Lectura de Datos Meteorológicos y de Flujo** { display-mode: "form" }
#hay dos archivos de mediciones largas sin objetivo
#'/content/drive/MyDrive/Pasantias UIS/Datos/Flujo(ST 3 1)/RA-31Oct22_100301.dat','2022-10-28 14:35:00'
#'/content/drive/MyDrive/Pasantias UIS/Datos/Flujo(ST 3 1)/RA-11Nov22_131444.dat','2022-11-08 08:59:00'
df_flujo=lectura('/content/drive/MyDrive/Pasantias UIS/Datos/Flujo(ST 3 1)/RA-11Nov22_131444.dat','2022-11-08 08:59:00')
df_met=lecturaMet('/content/drive/MyDrive/Pasantias UIS/Datos/Meteorologia (Davis)/29-31Oct.txt')

"""### Señal en cada centellador"""

#@title **Centellador Sup** { display-mode: "form" }
sns.set(rc = {'figure.figsize':(6,4)})
df_flujo3f=df_flujo[df_flujo['Ch3Cnts']<20000000]#Filtrado de Excesos
df_flujo3fmean=df_flujo3f.rolling(window=15).mean() #Rolling Window mean 
df_flujo3fmean['Ch3Cnts'].plot()
plt.ylabel('Flujo en el Cent. Sup RW',size=16)
plt.tick_params(labelsize=14)
plt.title('Modulación Diaria del Flujo Individual, Ch1',backgroundcolor='lightseagreen',color='w',size=16)
plt.show()
sns.set(rc = {'figure.figsize':(6,4)})
df_flujo3f['Ch3Cnts'].hist(bins=60)

#@title **Centellador Med** { display-mode: "form" }
sns.set(rc = {'figure.figsize':(6,4)})
df_flujo2f=df_flujo[df_flujo['Ch2Cnts']<6500]#Filtrado de Excesos
df_flujo2fmean=df_flujo2f.rolling(window=5).mean() #Rolling Window mean 
df_flujo2fmean['Ch2Cnts'].plot()
plt.ylabel('Flujo en el Cent. Med',size=16)
plt.tick_params(labelsize=14)
plt.title('Modulación Diaria del Flujo Individual, Ch2',backgroundcolor='lightseagreen',color='w',size=16)
plt.show()
sns.set(rc = {'figure.figsize':(6,4)})
df_flujo2f['Ch2Cnts'].hist(bins=20)

#@title **Centellador Inf** { display-mode: "form" }
sns.set(rc = {'figure.figsize':(6,4)})
df_flujo1f=df_flujo[df_flujo['Ch1Cnts']<6500]#Filtrado de Excesos (En este caso no hace falta )
df_flujo1fmean=df_flujo1f.rolling(window=5).mean() #Rolling Window mean 
df_flujo1fmean['Ch1Cnts'].plot()
plt.ylabel('Flujo en el Cent. Inf',size=16)
plt.tick_params(labelsize=14)
plt.title('Modulación Diaria del Flujo Individual, Ch1',backgroundcolor='lightseagreen',color='w',size=16)
plt.show()
sns.set(rc = {'figure.figsize':(6,4)})
df_flujo1f['Ch1Cnts'].hist(bins=20)

#@title **Coincidencias** { display-mode: "form" }
sns.set(rc = {'figure.figsize':(6,4)})
df_flujo0f=df_flujo[df_flujo['TrigCnts']<10500]#Filtrado de Excesos (En este caso no hace falta )
df_flujo0fmean=df_flujo1f.rolling(window=5).mean() #Rolling Window mean 
df_flujo0fmean['TrigCnts'].plot()
plt.ylabel('Flujo de Coincidencias',size=16)
plt.tick_params(labelsize=14)
plt.title('Modulación Diaria del Flujo Individual, Coincidencia',backgroundcolor='lightseagreen',color='w',size=16)
plt.show()
sns.set(rc = {'figure.figsize':(6,4)})

### ajuste de Poisson
a,b,c=plt.hist(df_flujo1f['TrigCnts'],bins=20,density=True)
a=np.concatenate(([0],a),axis=0)


def fit_function(k, lamb):
    '''poisson function, parameter lamb is the fit parameter'''
    return poisson.pmf(k, lamb)


# fit with curve_fit
parameters, cov_matrix = curve_fit(fit_function,b, a)
parameters
# plot poisson-deviation with fitted parameter
plt.plot(
    b,
    fit_function(b,*parameters)*a.sum(),linestyle='-.',
    label='Fit result',
)


plt.legend()
plt.show()
print(chi_cuadrado(a,fit_function(b,*parameters)))

"""### Modulacion Atmosferica

"""

#@title Correlaciones
df_flujo=lectura('/content/drive/MyDrive/Pasantias UIS/Datos/Flujo(ST 3 1)/RA-31Oct22_100301.dat','2022-10-28 14:35:00')
df=pd.DataFrame()
df['Flujo1']=df_flujo.Ch1Cnts['2022-10-28 14:36:00':'2022-10-31 10:00:00']
df['Flujo2']=df_flujo.Ch2Cnts['2022-10-28 14:36:00':'2022-10-31 10:00:00']
df['Flujo3']=df_flujo.Ch3Cnts['2022-10-28 14:36:00':'2022-10-31 10:00:00']
df['Flujo']=df_flujo.TrigCnts['2022-10-28 14:36:00':'2022-10-31 10:00:00']
df['Presion']=df_met.Bar
df['Hora']=df_met.Hora
df=df['2022-10-29 00:01:00':]
sns.pairplot(df)
df.corr()

#@title Correlaciones Ch2 y Presion
sns.scatterplot(data=df,x='Presion',y=df['Flujo1']/df['Flujo1'].mean(), s=100,alpha=0.2)
plt.ylabel('Flujo Norm en el Cent. Sup',size=16)
plt.yticks(size=12)
plt.xlabel('Presión [mm Hg]',size=16)
plt.title('Correlación Ch1 Vs Presión $[0.43]$',backgroundcolor='lightseagreen',color='w',size=16)
plt.ylim(0.8,1.3)
plt.show()

#@title Correlaciones Coincidencias y Presion
sns.scatterplot(data=df,x='Presion',y=df['Flujo']/df['Flujo'].mean(),s=100,alpha=0.2,color='green')
plt.ylabel('Flujo Norm Coincidencias',size=16)
plt.yticks(size=12)
plt.xlabel('Presión [mm Hg]',size=16)
plt.title('Correlación Coincidencias Vs Presión $[0.03]$',backgroundcolor='lightseagreen',color='w',size=16)
plt.ylim(0,3)
plt.show()

df_met.Bar.plot()
plt.title('Monitoreo de la Presión',backgroundcolor='lightseagreen',color='w',size=16)
plt.ylabel('Presión [mm Hg]',size=16)
plt.xticks(size=12)

"""## Comparación entre medidas """

#@title **Análisis PDF del %de atenuación y ajuste (Hierro)** { display-mode: "form" }
#Este es un código que calcula el ajuste de la PDF para el % Atenuacion para el Hierro
#Los ejes y bines estan ajustados con un fin estetico de una data particular, de cambiarla hacer los cambios respectivos
#los archivos de ST con Objetivos son los siguientes:
#/content/drive/MyDrive/Pasantias UIS/Datos/Flujo(ST 3 1)/RA-11Nov22_131444.dat (hierro)
#/content/drive/MyDrive/Pasantias UIS/Datos/Flujo(ST 3 1)/RA-02Nov22_161736.dat (Arena Seca)
#yeso en polvo
#Ladrillo de arcilla refractaria
#Granito (Gravilla)
# Lectura de Datos
df_sinObjeto=lectura('Datos/Flujo(ST 3 1)/RA-31Oct22_100301.dat','2022-10-28 14:35:00') #sin objeto
df_ConObjeto=lectura('Datos/Flujo(ST 3 1)/RA-06Nov22_155505.dat','2022-11-03 12:22:00') #hierro


#filtrado y Solapamiento Temporal
inicio1='2022-10-29 00:00:00'
final1='2022-10-31 00:00:00'

inicio2='2022-11-04 00:00:00'
final2='2022-11-06 00:00:00'


f1=df_sinObjeto[inicio1:final1]
g1=df_ConObjeto[inicio2:final2]

idxcomp = pd.date_range('00:00:00', periods = len(f1), freq ='min') 
f1.index=idxcomp
f1['time']=idxcomp

g1.index=idxcomp
g1['time']=idxcomp

df=pd.DataFrame()
df['Objeto']=g1['TrigCnts']
df['Sin']=f1['TrigCnts']
df['time']=idxcomp

#filtrado de eventos
df=df[df['Objeto']<(df['Objeto'].mean()+2*df['Objeto'].std())]

# Agrupamiento
df1=df.groupby(pd.Grouper(key='time', axis=0, freq='1min')).sum()
df1['%']=(100-df1['Objeto']*100/df1['Sin'])

# modificando l
#Promedio.append(df1['%'].mean())
#Error.append(df1['%'].std())

# filtrado de Nan y % desproporcionados
df1.fillna(0)
df1=df1[df1['%']>-100]

#ventaneo movil
dfr=df1.rolling(window=45).mean()
dfr.fillna(0)
dfr=dfr[45:]

# Ajuste distribución normal
#===============================================================================
# 1) Se define el tipo de distribución
distribucion = stats.norm
dataobs=dfr['%']

# 2) Con el método fit() se obtienen los parámetros
parametros = distribucion.fit(data=dataobs,method='MLE')

# 3) Se crea un diccionario que incluya el nombre de cada parámetro
nombre_parametros = [p for p in inspect.signature(distribucion._pdf).parameters \
                     if not p=='x'] + ["loc","scale"]
parametros_dict = dict(zip(nombre_parametros, parametros))

# 3) Se calcula el log likelihood
log_likelihood = distribucion.logpdf(dataobs.to_numpy(), *parametros).sum()

# 4) Se calcula el AIC y el BIC
aic = -2 * log_likelihood + 2 * len(parametros)
bic = -2 * log_likelihood + np.log(dataobs.shape[0]) * len(parametros)

# 5) Gráfico
sns.set(rc = {'figure.figsize':(8,6)})
sns.set_style("whitegrid",{"grid.color": ".6", "grid.linestyle": ":"})
x_hat = np.linspace(min(dataobs), max(dataobs), num=90)
y_hat = distribucion.pdf(x_hat, *parametros)
fig, ax = plt.subplots(figsize=(7,4))
ax.plot(x_hat, y_hat, linewidth=3, label='Ajuste',color='k')
ax.hist(x=dataobs, density=True, bins=18, color="#3182bd", alpha=0.5,label='Hist. %')
#ax.plot(dataobs, np.full_like(dataobs, -0.01), 'k', markeredgewidth=1)
#ax.vlines(parametros[0]-parametros[1],0,0.07, colors='r',linewidth=3,linestyle='--')
#ax.vlines(parametros[0]+parametros[1],0,0.07,colors='r',linewidth=3,linestyle='--')
ax.vlines(parametros[0]-2*parametros[1],0,0.041,colors='r',linewidth=3,linestyle='--',label='2$\sigma$')
ax.vlines(parametros[0]+2*parametros[1],0,0.041,colors='r',linewidth=3,linestyle='--')
ax.set_title('Ajuste MLE', size=18, backgroundcolor='SteelBlue',color='w')
ax.set_xlabel('% de atenuación del Flujo',size=16)
ax.set_ylabel('Densidad de probabilidad',size=16)
ax.legend(facecolor='grey', framealpha=0.4,fontsize=16)
x = [15,25,45,parametros[0]-parametros[1],parametros[0]+parametros[1],parametros[0]-2*parametros[1],parametros[0]+2*parametros[1]]
plt.tick_params(labelsize=14)
val = ['15','25','45','$\sigma$','$\sigma$','$2\sigma$','$2\sigma$']
plt.xticks(x,val)
#6) Información del ajuste

y=np.histogram(dataobs,  bins=len(x_hat),density=True)

print('---------------------')
print('Resultados del ajuste')
print('---------------------')
print(f"Distribución:   {distribucion.name}")
print(f"Dominio:        {[distribucion.a, distribucion.b]}")
print(f"Parámetros:     {parametros_dict}")
print(f"Log likelihood: {log_likelihood}")
print(f"AIC:            {aic}")
print(f"BIC:            {bic}")
print(f"Chi_Cuadrado:      ", chi_cuadrado(y[0], y_hat))
plt.plot()

#@title **Análisis PDF del %de atenuación y ajuste (Arena)** { display-mode: "form" }
#Este es un código que calcula el ajuste de la PDF para el % Atenuacion para el Hierro
#Los ejes y bines estan ajustados con un fin estetico de una data particular, de cambiarla hacer los cambios respectivos
#los archivos de ST con Objetivos son los siguientes:
#/content/drive/MyDrive/Pasantias UIS/Datos/Flujo(ST 3 1)/RA-11Nov22_131444.dat (hierro)
#/content/drive/MyDrive/Pasantias UIS/Datos/Flujo(ST 3 1)/RA-02Nov22_161736.dat (Arena Seca)
#yeso en polvo
#Ladrillo de arcilla refractaria
#Granito (Gravilla)
# Lectura de Datos
df_sinObjeto=lectura('Datos/Flujo(ST 3 1)/RA-11Nov22_131444.dat','2022-11-08 08:59:00') #sin objeto
df_ConObjeto=lectura('Datos/Flujo(ST 3 1)/RA-02Nov22_161736.dat','2022-11-01 14:49:00') #Arena


#filtrado y Solapamiento Temporal
inicio1='2022-11-08 14:50:00'
final1='2022-11-09 16:15:00'

inicio2='2022-11-01 14:50:00'
final2='2022-11-02 16:15:00'


f1=df_sinObjeto[inicio1:final1]
g1=df_ConObjeto[inicio2:final2]

idxcomp = pd.date_range('00:00:00', periods = len(f1), freq ='min') 
f1.index=idxcomp
f1['time']=idxcomp

g1.index=idxcomp
g1['time']=idxcomp

df=pd.DataFrame()
df['Objeto']=g1['TrigCnts']
df['Sin']=f1['TrigCnts']
df['time']=idxcomp

#filtrado de eventos
df=df[df['Objeto']<(df['Objeto'].mean()+2*df['Objeto'].std())]

# Agrupamiento
df1=df.groupby(pd.Grouper(key='time', axis=0, freq='1min')).sum()
df1['%']=(100-df1['Objeto']*100/df1['Sin'])

#Promedio.append(df1['%'].mean())
#Error.append(df1['%'].std())
#dfV=pd.DataFrame()
#dfV['Prom']=Promedio
#dfV['Std']=Error
#dfV['Ventana']=np.arange(2,122,2)
#dfV.to_csv('datosResolucion1[tornillos].txt', sep=" ",
#          quoting=csv.QUOTE_NONE, escapechar=" ")

# filtrado de Nan y % desproporcionados
df1.fillna(0)
df1=df1[df1['%']>-60]

#ventaneo movil
dfr=df1.rolling(window=65).mean()
dfr.fillna(0)
dfr=dfr[65:]

# Ajuste distribución normal
#===============================================================================
# 1) Se define el tipo de distribución
distribucion = stats.norm
dataobs=dfr['%']

# 2) Con el método fit() se obtienen los parámetros
parametros = distribucion.fit(data=dataobs,method='MLE')

# 3) Se crea un diccionario que incluya el nombre de cada parámetro
nombre_parametros = [p for p in inspect.signature(distribucion._pdf).parameters \
                     if not p=='x'] + ["loc","scale"]
parametros_dict = dict(zip(nombre_parametros, parametros))

# 3) Se calcula el log likelihood
log_likelihood = distribucion.logpdf(dataobs.to_numpy(), *parametros).sum()

# 4) Se calcula el AIC y el BIC
aic = -2 * log_likelihood + 2 * len(parametros)
bic = -2 * log_likelihood + np.log(dataobs.shape[0]) * len(parametros)

# 5) Gráfico
x_hat = np.linspace(min(dataobs), max(dataobs), num=20)
y_hat = distribucion.pdf(x_hat, *parametros)
fig, ax = plt.subplots(figsize=(7,4))
ax.plot(x_hat, y_hat, linewidth=3, label='Ajuste',color='k')
ax.hist(x=dataobs, density=True, bins=18, color="#3182bd", alpha=0.5,label='Hist. %')
ax.plot(dataobs, np.full_like(dataobs, -0.01), '|k', markeredgewidth=1)
ax.vlines(parametros[0]-parametros[1],0,0.07, colors='r',linewidth=3,linestyle='--')
ax.vlines(parametros[0]+parametros[1],0,0.07,colors='r',linewidth=3,linestyle='--')
ax.vlines(parametros[0]-2*parametros[1],0,0.03,colors='r',linewidth=3,linestyle='--')
ax.vlines(parametros[0]+2*parametros[1],0,0.03,colors='r',linewidth=3,linestyle='--')
ax.set_title('Ajuste Gaussiano por MLE', size=18, backgroundcolor='SteelBlue',color='w')
ax.set_xlabel('% de atenuación del Flujo',size=16)
ax.set_ylabel('Densidad de probabilidad',size=16)
ax.legend(facecolor='grey', framealpha=0.9,fontsize=16)
#x = [15,20,30,40,45,parametros[0]-parametros[1],parametros[0]+parametros[1],parametros[0]-2*parametros[1],parametros[0]+2*parametros[1]]
plt.tick_params(labelsize=14)
#val = ['15','20','30','40','45','$\sigma$','$\sigma$','$2\sigma$','$2\sigma$']
#plt.xticks(x,val)
#6) Información del ajuste

y=np.histogram(dataobs,  bins=len(x_hat),density=True)

print('---------------------')
print('Resultados del ajuste')
print('---------------------')
print(f"Distribución:   {distribucion.name}")
print(f"Dominio:        {[distribucion.a, distribucion.b]}")
print(f"Parámetros:     {parametros_dict}")
print(f"Log likelihood: {log_likelihood}")
print(f"AIC:            {aic}")
print(f"BIC:            {bic}")
print(f"Chi_Cuadrado:      ", chi_cuadrado(y[0], y_hat))
plt.plot()

#@title **Análisis PDF del %de atenuación y ajuste (Yesso)** { display-mode: "form" }
#Este es un código que calcula el ajuste de la PDF para el % Atenuacion para el Hierro
#Los ejes y bines estan ajustados con un fin estetico de una data particular, de cambiarla hacer los cambios respectivos
#los archivos de ST con Objetivos son los siguientes:
#/content/drive/MyDrive/Pasantias UIS/Datos/Flujo(ST 3 1)/RA-11Nov22_131444.dat (hierro)
#/content/drive/MyDrive/Pasantias UIS/Datos/Flujo(ST 3 1)/RA-02Nov22_161736.dat (Arena Seca)
#yeso en polvo
#Ladrillo de arcilla refractaria
#Granito (Gravilla)
# Lectura de Datos
df_sinObjeto=lectura('Datos/Flujo(ST 3 1)/RA-11Nov22_131444.dat','2022-11-08 08:59:00') #sin objeto
df_ConObjeto=lectura('Datos/Flujo(ST 3 1)/RA-18Nov22_091404.dat','2022-11-15 17:44:00') #Yesso


#filtrado y Solapamiento Temporal
inicio1='2022-11-08 14:50:00'
final1='2022-11-10 08:55:00'

inicio2='2022-11-16 14:50:00'
final2='2022-11-18 08:55:00'


f1=df_sinObjeto[inicio1:final1]
g1=df_ConObjeto[inicio2:final2]

idxcomp = pd.date_range('00:00:00', periods = len(f1), freq ='min') 
f1.index=idxcomp
f1['time']=idxcomp

g1.index=idxcomp
g1['time']=idxcomp

df=pd.DataFrame()
df['Objeto']=g1['TrigCnts']
df['Sin']=f1['TrigCnts']
df['time']=idxcomp

#filtrado de eventos
df=df[df['Objeto']<(df['Objeto'].mean()+2*df['Objeto'].std())]

# Agrupamiento
df1=df.groupby(pd.Grouper(key='time', axis=0, freq='1min')).sum()
df1['%']=(100-df1['Objeto']*100/df1['Sin'])

#Promedio.append(df1['%'].mean())
#Error.append(df1['%'].std())
#dfV=pd.DataFrame()
#dfV['Prom']=Promedio
#dfV['Std']=Error
#dfV['Ventana']=np.arange(2,122,2)
#dfV.to_csv('datosResolucion1[tornillos].txt', sep=" ",
#          quoting=csv.QUOTE_NONE, escapechar=" ")

# filtrado de Nan y % desproporcionados
df1.fillna(0)
df1=df1[df1['%']>-100]

#ventaneo movil
dfr=df1.rolling(window=60).mean()
dfr.fillna(0)
dfr=dfr[60:]

# Ajuste distribución normal
#===============================================================================
# 1) Se define el tipo de distribución
distribucion = stats.norm
dataobs=dfr['%']

# 2) Con el método fit() se obtienen los parámetros
parametros = distribucion.fit(data=dataobs,method='MLE')

# 3) Se crea un diccionario que incluya el nombre de cada parámetro
nombre_parametros = [p for p in inspect.signature(distribucion._pdf).parameters \
                     if not p=='x'] + ["loc","scale"]
parametros_dict = dict(zip(nombre_parametros, parametros))

# 3) Se calcula el log likelihood
log_likelihood = distribucion.logpdf(dataobs.to_numpy(), *parametros).sum()

# 4) Se calcula el AIC y el BIC
aic = -2 * log_likelihood + 2 * len(parametros)
bic = -2 * log_likelihood + np.log(dataobs.shape[0]) * len(parametros)

# 5) Gráfico
x_hat = np.linspace(min(dataobs), max(dataobs), num=20)
y_hat = distribucion.pdf(x_hat, *parametros)
fig, ax = plt.subplots(figsize=(7,4))
ax.plot(x_hat, y_hat, linewidth=3, label='Ajuste',color='k')
ax.hist(x=dataobs, density=True, bins=18, color="#3182bd", alpha=0.5,label='Hist. %')
ax.plot(dataobs, np.full_like(dataobs, -0.01), '|k', markeredgewidth=1)
ax.vlines(parametros[0]-parametros[1],0,0.07, colors='r',linewidth=3,linestyle='--')
ax.vlines(parametros[0]+parametros[1],0,0.07,colors='r',linewidth=3,linestyle='--')
ax.vlines(parametros[0]-2*parametros[1],0,0.03,colors='r',linewidth=3,linestyle='--')
ax.vlines(parametros[0]+2*parametros[1],0,0.03,colors='r',linewidth=3,linestyle='--')
ax.set_title('Ajuste Gaussiano por MLE', size=18, backgroundcolor='SteelBlue',color='w')
ax.set_xlabel('% de atenuación del Flujo',size=16)
ax.set_ylabel('Densidad de probabilidad',size=16)
ax.legend(facecolor='grey', framealpha=0.9,fontsize=16)
#x = [15,20,30,40,45,parametros[0]-parametros[1],parametros[0]+parametros[1],parametros[0]-2*parametros[1],parametros[0]+2*parametros[1]]
plt.tick_params(labelsize=14)
#val = ['15','20','30','40','45','$\sigma$','$\sigma$','$2\sigma$','$2\sigma$']
#plt.xticks(x,val)
#6) Información del ajuste

y=np.histogram(dataobs,  bins=len(x_hat),density=True)

print('---------------------')
print('Resultados del ajuste')
print('---------------------')
print(f"Distribución:   {distribucion.name}")
print(f"Dominio:        {[distribucion.a, distribucion.b]}")
print(f"Parámetros:     {parametros_dict}")
print(f"Log likelihood: {log_likelihood}")
print(f"AIC:            {aic}")
print(f"BIC:            {bic}")
print(f"Chi_Cuadrado:      ", chi_cuadrado(y[0], y_hat))
plt.plot()

a=[[26.208947566735915,6.670202438951552,3.56],[13.40,  6.245 ,1.22],[9.242414293754354,7.024525406849215,1.08]]
dffinal=pd.DataFrame(a, columns=['%','std','Densidad'])
dffinal
dffinal.plot(x='Densidad',y='%',yerr='std', kind='scatter',fontsize=16,linewidth=6,color='k',s=24)
plt.ylabel('% de atenuación del flujo', size=16)
plt.xlabel('Densidad del objetivo $[g \cdot cm^{-3}]$', size=16)
plt.title('Atenuación del Flujo vs Densidad', size=18, backgroundcolor='SteelBlue',color='w')